@@ -77,7 +77,6 @@ enum
 
 static void            terminal_window_dispose                       (GObject                *object);
 static void            terminal_window_finalize                      (GObject                *object);
-static void            terminal_window_realize                       (GtkWidget              *widget);
 static gboolean        terminal_window_delete_event                  (GtkWidget              *widget,
                                                                       GdkEventAny            *event);
 static gboolean        terminal_window_state_event                   (GtkWidget              *widget,
@@ -203,9 +202,6 @@ struct _TerminalWindow
   GtkWidget           *notebook;
 
   TerminalScreen      *active;
-
-  /* whether this window has an rgba colormap */
-  guint                is_composited : 1;
 };
 
 
@@ -270,7 +266,6 @@ terminal_window_class_init (TerminalWindowClass *klass)
   gobject_class->finalize = terminal_window_finalize;
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
-  gtkwidget_class->realize = terminal_window_realize;
   gtkwidget_class->window_state_event = terminal_window_state_event;
   gtkwidget_class->delete_event = terminal_window_delete_event;
   gtkwidget_class->style_set = terminal_window_style_set;
@@ -315,9 +310,17 @@ terminal_window_init (TerminalWindow *window)
   gboolean        always_show_tabs;
   gchar          *role;
   GtkRcStyle     *style;
+  GdkScreen      *screen;
+  GdkColormap    *colormap;
 
   window->preferences = terminal_preferences_get ();
 
+  /* try to set the rgba colormap so vte can use real transparency */
+  screen = gtk_window_get_screen (GTK_WINDOW (window));
+  colormap = gdk_screen_get_rgba_colormap (screen);
+  if (colormap != NULL)
+    gtk_widget_set_colormap (GTK_WIDGET (window), colormap);
+
   /* The Terminal size needs correction when the font name or the scrollbar
    * visibility is changed.
    */
@@ -438,34 +441,6 @@ terminal_window_finalize (GObject *object)
 
 
 
-static void
-terminal_window_realize (GtkWidget *widget)
-{
-  TerminalWindow *window = TERMINAL_WINDOW (widget);
-  GdkScreen      *screen;
-  GdkColormap    *colormap;
-
-  /* unset rgba */
-  window->is_composited = FALSE;
-
-  /* try to set the rgba colormap so vte can use real transparency */
-  screen = gtk_widget_get_screen (widget);
-  if (gdk_screen_is_composited (screen))
-    {
-      colormap = gdk_screen_get_rgba_colormap (screen);
-      if (G_LIKELY (colormap != NULL))
-        window->is_composited = TRUE;
-      else
-        colormap = gdk_screen_get_default_colormap (screen);
-
-      gtk_widget_set_colormap (widget, colormap);
-    }
-
-  (*GTK_WIDGET_CLASS (terminal_window_parent_class)->realize) (widget);
-}
-
-
-
 static gboolean
 terminal_window_delete_event (GtkWidget   *widget,
                               GdkEventAny *event)
@@ -1520,35 +1495,46 @@ terminal_window_action_goto_tab (GtkRadioAction *action,
 
 
 static void
-title_dialog_response (GtkWidget *dialog,
-                       gint       response)
+title_dialog_response (GtkWidget         *dialog,
+                       gint              response,
+                       ExoMutualBinding *title_binding)
 {
   /* check if we should open the user manual */
   if (response == GTK_RESPONSE_HELP)
     {
       /* open the "Set Title" paragraph in the "Usage" section */
-      terminal_dialogs_show_help (GTK_WIDGET (dialog), "usage", "set-title");
+      terminal_dialogs_show_help (GTK_WINDOW (dialog), "usage.html", "set-title");
     }
   else
     {
       /* close the dialog */
+      exo_mutual_binding_unbind (title_binding);
       gtk_widget_destroy (dialog);
     }
 }
 
 
 
+static void
+title_dialog_quit (GtkWidget *dialog)
+{
+  gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+}
+
+
+
 static void
 terminal_window_action_set_title (GtkAction      *action,
                                   TerminalWindow *window)
 {
-  AtkRelationSet *relations;
-  AtkRelation    *relation;
-  AtkObject      *object;
-  GtkWidget      *dialog;
-  GtkWidget      *box;
-  GtkWidget      *label;
-  GtkWidget      *entry;
+  AtkRelationSet   *relations;
+  AtkRelation      *relation;
+  AtkObject        *object;
+  GtkWidget        *dialog;
+  GtkWidget        *box;
+  GtkWidget        *label;
+  GtkWidget        *entry;
+  ExoMutualBinding *title_binding;
 
   if (G_LIKELY (window->active != NULL))
     {
@@ -1572,7 +1558,7 @@ terminal_window_action_set_title (GtkAction      *action,
       entry = gtk_entry_new ();
       gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
       g_signal_connect_swapped (G_OBJECT (entry), "activate",
-                                G_CALLBACK (gtk_widget_destroy), dialog);
+                                G_CALLBACK (title_dialog_quit), dialog);
       gtk_widget_show (entry);
 
       /* set Atk description and label relation for the entry */
@@ -1583,10 +1569,11 @@ terminal_window_action_set_title (GtkAction      *action,
       atk_relation_set_add (relations, relation);
       g_object_unref (G_OBJECT (relation));
 
-      exo_mutual_binding_new (G_OBJECT (window->active), "custom-title", G_OBJECT (entry), "text");
+      title_binding = exo_mutual_binding_new (G_OBJECT (window->active), "custom-title",
+                                              G_OBJECT (entry), "text");
 
       g_signal_connect (G_OBJECT (dialog), "response",
-                        G_CALLBACK (title_dialog_response), NULL);
+                        G_CALLBACK (title_dialog_response), title_binding);
 
       gtk_widget_show (dialog);
     }
@@ -1619,7 +1606,7 @@ terminal_window_action_report_bug (GtkAction       *action,
                                    TerminalWindow  *window)
 {
   /* open the "Support" section of the user manual */
-  terminal_dialogs_show_help (GTK_WIDGET (window), "support", NULL);
+  terminal_dialogs_show_help (GTK_WINDOW (window), "support.html", NULL);
 }
 
 
@@ -1629,7 +1616,7 @@ terminal_window_action_contents (GtkAction       *action,
                                  TerminalWindow  *window)
 {
   /* open the Terminal user manual */
-  terminal_dialogs_show_help (GTK_WIDGET (window), NULL, NULL);
+  terminal_dialogs_show_help (GTK_WINDOW (window), NULL, NULL);
 }
 
 
